@@ -45,6 +45,7 @@
 #include <freetds/stream.h>
 #include <freetds/convert.h>
 #include <freetds/utils/string.h>
+#include <freetds/utils/ascii.h>
 #include <freetds/replacements.h>
 #include <freetds/enum_cap.h>
 
@@ -376,6 +377,17 @@ tds7_build_bulk_insert_stmt(TDSSOCKET * tds, TDSPBCB * clause, TDSCOLUMN * bcpco
 	return TDS_SUCCESS;
 }
 
+static const char *
+uc_str(const char *s)
+{
+	size_t n = strcspn(s, "abcdefghijklmnopqrstuvwxyz");
+
+	if (s[n] == '\0')
+		return s;	/* already all uppercase */
+
+	return tds_ascii_strupr(strdup(s));
+}
+
 /**
  * Prepare the query to be sent to server to request BCP information
  * \tds
@@ -392,6 +404,7 @@ tds_bcp_start_insert_stmt(TDSSOCKET * tds, TDSBCPINFO * bcpinfo)
 		TDSCOLUMN *bcpcol;
 		TDSPBCB colclause;
 		char clause_buffer[4096] = { 0 };
+		bool triggers_checked = tds_dstr_isempty(&bcpinfo->hint);	/* Done on demand */
 
 		colclause.pb = clause_buffer;
 		colclause.cb = sizeof(clause_buffer);
@@ -407,8 +420,21 @@ tds_bcp_start_insert_stmt(TDSSOCKET * tds, TDSBCPINFO * bcpinfo)
 				continue;
 			if (!bcpinfo->identity_insert_on && bcpcol->column_identity)
 				continue;
-			if (bcpcol->column_computed)
-				continue;
+			if (bcpcol->column_computed) {
+				if (!triggers_checked) {
+					const char *uc_hint = uc_str(tds_dstr_cstr(&bcpinfo->hint));
+
+					if (strstr(uc_hint, "FIRE_TRIGGERS"))
+						bcpinfo->with_triggers = true;
+
+					if (uc_hint != tds_dstr_cstr(&bcpinfo->hint))
+						free((char *) uc_hint);
+
+					triggers_checked = true;
+				}
+				if (!bcpinfo->with_triggers)
+					continue;
+			}
 			tds7_build_bulk_insert_stmt(tds, &colclause, bcpcol, firstcol);
 			firstcol = 0;
 		}
@@ -467,9 +493,8 @@ tds7_send_record(TDSSOCKET *tds, TDSBCPINFO *bcpinfo,
 		 * identity columns unless indentity_insert is enabled.
 		 */
 
-		if ((!bcpinfo->identity_insert_on && bindcol->column_identity) ||
-			bindcol->column_timestamp ||
-			bindcol->column_computed) {
+		if ((!bcpinfo->identity_insert_on && bindcol->column_identity) || bindcol->column_timestamp
+		    || (bindcol->column_computed && !bcpinfo->with_triggers)) {
 			continue;
 		}
 
@@ -500,7 +525,7 @@ tds7_send_record(TDSSOCKET *tds, TDSBCPINFO *bcpinfo,
 			bindcol->column_cur_size = bindcol->bcp_column_data->datalen;
 			bindcol->column_data = bindcol->bcp_column_data->data;
 		}
-		rc = bindcol->funcs->put_data(tds, bindcol, 1);
+		rc = bindcol->funcs->put_data(tds, bindcol, true);
 		bindcol->column_cur_size = save_size;
 		bindcol->column_data = save_data;
 
@@ -925,9 +950,8 @@ tds7_bcp_send_colmetadata(TDSSOCKET *tds, TDSBCPINFO *bcpinfo)
 	num_cols = 0;
 	for (i = 0; i < bcpinfo->bindinfo->num_cols; i++) {
 		bcpcol = bcpinfo->bindinfo->columns[i];
-		if ((!bcpinfo->identity_insert_on && bcpcol->column_identity) || 
-			bcpcol->column_timestamp ||
-			bcpcol->column_computed) {
+		if ((!bcpinfo->identity_insert_on && bcpcol->column_identity) || bcpcol->column_timestamp
+		    || (bcpcol->column_computed && !bcpinfo->with_triggers)) {
 			continue;
 		}
 		num_cols++;
@@ -946,9 +970,8 @@ tds7_bcp_send_colmetadata(TDSSOCKET *tds, TDSBCPINFO *bcpinfo)
 		 * identity columns (unless indentity_insert is enabled
 		 */
 
-		if ((!bcpinfo->identity_insert_on && bcpcol->column_identity) || 
-			bcpcol->column_timestamp ||
-			bcpcol->column_computed) {
+		if ((!bcpinfo->identity_insert_on && bcpcol->column_identity) || bcpcol->column_timestamp
+		    || (bcpcol->column_computed && !bcpinfo->with_triggers)) {
 			continue;
 		}
 
